@@ -842,6 +842,11 @@ class EdgeFilter:
             ndimage.binary_dilation(
                 source_mask, structure=np.ones((1, extension_extraction_height,
                                                 (2*extraction_width-16)//2)))
+        # remove possible small spurious structures
+        s = np.ones((2,2,1), dtype=bool)
+        source_mask = ndimage.binary_erosion(source_mask, structure=s)
+        s = np.ones((3,2,1), dtype=bool)
+        source_mask = ndimage.binary_dilation(source_mask, structure=s)
 
         s = ndimage.generate_binary_structure(3,2)
         labeled_source_mask, num_sources = ndimage.label(source_mask ,structure=s)
