@@ -16,16 +16,17 @@ from typing import Union
 from scipy.spatial.transform import Rotation as R
 from scipy.stats import multivariate_normal
 from astropy.convolution import Kernel2D
+from astropy.convolution import Gaussian2DKernel
 from astropy.modeling.models import Gaussian2D
 from astropy.modeling.parameters import Parameter
 from cascade_filtering.stencil import IMPLEMENTED_KERNEL_SIZES
 from cascade_filtering import __path__
 
-
-__all__ = ['FilterKernel', 'create_anisotropic_curved_kernels', 
+__all__ = ['FilterKernel', 'create_anisotropic_curved_kernels',
            'create_nagano_matsuyama_kernels', 'create_kuwahara_kernels',
-           'define_covariance_matrix', 'define_anisotropic_gaussian_kernel']
-
+           'define_covariance_matrix', 'define_anisotropic_gaussian_kernel',
+           'create_square_kernels', 'create_sift_kernels',
+           'create_gaussian_kernel']
 
 CONFIG_PATH = pathlib.Path(os.path.dirname(__path__[0])) / 'configuration_files/'
 
@@ -128,12 +129,12 @@ def define_covariance_matrix(sigma=[1.0, 1.0, 1.0], theta=[0.0], degrees=False):
     """
     Define covariance matrix.
 
-    Define 2D covariance matrix based on the standard deviation in the 
+    Define 2D covariance matrix based on the standard deviation in the
     dispersion and cross-dispersion direction, or the 3D covariance matrix
-    including the standard deviation in the time direction. Rotation angles 
+    including the standard deviation in the time direction. Rotation angles
     can be specified to indicate either a rotation in the dispersion -
     cross-dispersion plane (2D and 3D), or around all axis (3D).
-    
+
     Parameters
     ----------
     sigma : 'list' of float or int, or int or float
@@ -152,7 +153,7 @@ def define_covariance_matrix(sigma=[1.0, 1.0, 1.0], theta=[0.0], degrees=False):
         else:
             raise ValueError("Input sigma of wrong type")
     if len(sigma) == 1:
-        # simple symmetric 2D gaussian kernel 
+        # simple symmetric 2D gaussian kernel
         covariance_matrix = np.diag(sigma + sigma)**2
     elif len(sigma) in [2, 3]:
         # standard 2d or 3D kernel
@@ -160,13 +161,13 @@ def define_covariance_matrix(sigma=[1.0, 1.0, 1.0], theta=[0.0], degrees=False):
     else:
         # to many sigma's
         raise ValueError("input list of sigma's to long.")
- 
-    
+
+
     if not isinstance(theta, list):
         if isinstance(theta, (int, float)):
             theta = [theta]
         else:
-            raise ValueError("Input sigma of wrong type") 
+            raise ValueError("Input sigma of wrong type")
     if covariance_matrix.shape == (2,2):
         #  2D covariance with rotation
         if len(theta) == 1:
@@ -177,14 +178,14 @@ def define_covariance_matrix(sigma=[1.0, 1.0, 1.0], theta=[0.0], degrees=False):
     else:
         if len(theta) == 1:
         # rotation only in XY plain.
-            rotation_vector = [0, 0] + theta 
+            rotation_vector = [0, 0] + theta
         elif len(theta) == 3:
             rotation_vector = theta
         else:
             raise ValueError("defination of rotation angles not consistent "
                              "with 3D Kernel")
     rotation_matrix = define_rotation_matrix(rotation_vector, degrees=degrees)
-    rotation_matrix = rotation_matrix[-covariance_matrix.shape[0]:, 
+    rotation_matrix = rotation_matrix[-covariance_matrix.shape[0]:,
                                       -covariance_matrix.shape[1]:]
     covariance_matrix = np.linalg.multi_dot([rotation_matrix,
                                              covariance_matrix,
@@ -218,7 +219,7 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
                                        theta: Union[list, int, float],
                                        degrees=True) -> np.ndarray:
     """
-    
+    Define the anisitropic gaussian kernel.
 
     Parameters
     ----------
@@ -250,14 +251,14 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
         raise ValueError("kernel shape inconsistent with square or cube "
                              "shaped kernel.")
     kernel_size = kernel_shape[0]
-    
-    half_width = (kernel_size*n_oversample)//2    
+
+    half_width = (kernel_size*n_oversample)//2
 
     if not isinstance(sigma, list):
         if isinstance(sigma, (int, float)):
             sigma = [sigma]
         else:
-            raise ValueError("Input sigma of wrong type") 
+            raise ValueError("Input sigma of wrong type")
     oversampled_sigma = [mu*n_oversample for mu in sigma]
 
 
@@ -265,11 +266,11 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
 
         x = np.arange(-half_width, half_width+1)
         y = np.arange(-half_width, half_width+1)
-    
+
         X, Y = np.meshgrid(x,y, indexing='xy')
-    
+
         grid_positions = np.empty(X.shape + (2,))
-    
+
         grid_positions[..., 0] = Y
         grid_positions[..., 1] = X
 
@@ -282,9 +283,9 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
         x = np.arange(-half_width, half_width+1)
         y = np.arange(-half_width, half_width+1)
         z = np.arange(-half_width, half_width+1)
-    
+
         X, Y, Z = np.meshgrid(x,y,z, indexing='xy')
-    
+
         grid_positions = np.empty(X.shape + (3,))
         grid_positions[..., 0] = Y
         grid_positions[..., 1] = X
@@ -294,13 +295,13 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
                        n_oversample, kernel_size, n_oversample)
         rebin_axis_tuple = (1,3,5)
 
-    
+
     covariance_matrix = define_covariance_matrix(sigma=oversampled_sigma,
                                                  theta=theta,
                                                  degrees=degrees)
     center_position = np.zeros((kernel_dimension))
     rv = multivariate_normal(center_position, covariance_matrix)
-    
+
     kernel = rv.pdf(grid_positions)
     kernel = np.mean(kernel.reshape(rebin_tuple), axis=rebin_axis_tuple)
     kernel /= np.sum(kernel)
@@ -308,7 +309,7 @@ def define_anisotropic_gaussian_kernel(kernel_shape: tuple,
     return kernel
 
 
-def create_anisotropic_curved_kernels(kernel_size=9):
+def create_anisotropic_curved_kernels(kernel_size=9) -> np.ndarray:
     """
     Directional filters for smoothing and filtering.
 
@@ -328,7 +329,7 @@ def create_anisotropic_curved_kernels(kernel_size=9):
     kernel_stack : numpy.ndarray of 'float'
         Array containing all oriented filter kernels used for edge
         preserving smooting.
- 
+
     Notes
     -----
     When adding kernels, make sure the maximum is in the central pixel
@@ -339,7 +340,7 @@ def create_anisotropic_curved_kernels(kernel_size=9):
               np.radians(4.5), np.radians(-6.0), np.radians(6.0),
               np.radians(-9.0), np.radians(9.0), np.radians(-12.0),
               np.radians(12.0),
-              np.radians(0.0), 
+              np.radians(0.0),
               np.radians(90)-np.radians(60), np.radians(90)+np.radians(60),
               np.radians(90)-np.radians(60), np.radians(90)+np.radians(60)]
 
@@ -376,9 +377,128 @@ def create_anisotropic_curved_kernels(kernel_size=9):
     return kernel_stack
 
 
+def create_gaussian_kernel(kernel_size:int = 7, sigma:float = 1.0):
+    """
+    Create gaussian kernel.
+
+    Parameters
+    ----------
+    kernel_size : int, optional
+        DESCRIPTION. The default is 7.
+    sigma : float, optional
+        DESCRIPTION. The default is 1.0.
+
+    Returns
+    -------
+    TYPE
+        DESCRIPTION.
+
+    """
+    kernel = Gaussian2DKernel(x_stddev=sigma, y_stddev=sigma,
+                              x_size=kernel_size, y_size=kernel_size)
+    kernel.normalize()
+    return kernel.array
+
+
+def create_sift_kernels(kernel_size:int = 11) -> Union[np.ndarray, float, float]:
+    """
+    Create a kernel stack for SIFT.
+
+    Parameters
+    ----------
+    kernel_size : int, optional
+        DESCRIPTION. The default is 11.
+
+    Returns
+    -------
+    kernel_stack : TYPE
+        DESCRIPTION.
+    s : TYPE
+        DESCRIPTION.
+    min_sigma : TYPE
+        DESCRIPTION.
+
+    """
+    min_sigma = 0.5
+    max_sigma = kernel_size/6.0
+    n_kernels = kernel_size
+    s = (max_sigma/min_sigma)**(1.0/(n_kernels-1))
+    sigmas = [min_sigma*s**i for i in range(n_kernels)]
+
+    kernel_stack = []
+    for ik in range(n_kernels):
+        kernel = Gaussian2DKernel(x_stddev=sigmas[ik], y_stddev=sigmas[ik],
+                                  x_size=kernel_size, y_size=kernel_size)
+        kernel.normalize()
+        kernel_stack.append(kernel.array)
+    kernel_stack = np.array(kernel_stack)
+    return kernel_stack, s, min_sigma
+
+
+def create_sobel_kernels(kernel_size : int = 3):
+    if kernel_size == 3:
+        Gx = np.array([[-1, 0, 1], [-2, 0, 2], [-1, 0, 1]], dtype=float)/8.
+        Gy = np.array([[-1, -2, -1], [0, 0, 0], [1, 2, 1]], dtype=float)/8.
+    else:
+        Gx = np.array([[-5, -4, 0, 4, 5], [-8, -10, 0, 10, 8],
+                       [-10, -20, 0, 20, 10], [-8, -10, 0, 10, 8],
+                       [-5, -4, 0, 4, 5]], dtype=float)/240.
+        Gy = np.array([[-5, -8, -10, -8, -5], [-4, -10, -20, -10, -4],
+                       [0, 0, 0, 0, 0], [4, 10, 20, 10, 4],
+                       [5, 8, 10, 8, 5]], dtype=float)/240.
+    kernel_stack = []
+    kernel_stack.append(Gx)
+    kernel_stack.append(Gy)
+    kernel_stack = np.array(kernel_stack)
+    return kernel_stack
+
+
+def create_schar_kernels(kernel_size : int = 3):
+    if kernel_size == 3:
+        Gx = np.array([[-1, 0, 1], [-3, 0, 3], [-1, 0, 1]], dtype=float)/10.
+        Gy = np.array([[-1, -3, -1], [0, 0, 0], [1, 3, 1]], dtype=float)/10.
+    else:
+        Gx = np.array([[-1, -1, 0, 1, 1], [-2, -2, 0, 2, 2],
+                       [-3, -6, 0, 6, 3], [-2, -2, 0, 2, 2],
+                       [-1, -1, 0, 1, 1]], dtype=float)/60.
+        Gy = np.array([[-1, -2, -3, -2, -1], [-1, -2, -6, -2, -1],
+                       [0, 0, 0, 0, 0], [1, 2, 6, 2, 1],
+                       [1, 2, 3, 2, 1]], dtype=float)/60.
+    kernel_stack = []
+    kernel_stack.append(Gx)
+    kernel_stack.append(Gy)
+    kernel_stack = np.array(kernel_stack)
+    return kernel_stack
+
+
+def create_square_kernels(kernel_size=11) -> np.ndarray:
+    """
+    Create stack of square kernels.
+
+    Parameters
+    ----------
+    kernel_size : TYPE, optional
+        Kernel size. The default is 9.
+
+    Returns
+    -------
+    kernel_stack: 'ndarray'
+        Kernel stack
+
+    """
+    kernel_stack = []
+    for i in range(kernel_size//2):
+        kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
+        kernel_temp[i:kernel_size-i, i:kernel_size-i] = 1.0
+        kernel_stack.append(kernel_temp)
+    kernel_stack = np.array(kernel_stack)
+
+    return kernel_stack
+
+
 def create_kuwahara_kernels(kernel_size=9) -> np.ndarray:
     """
-    
+    Create stack of Kuwahara kernels.
 
     Parameters
     ----------
@@ -391,15 +511,10 @@ def create_kuwahara_kernels(kernel_size=9) -> np.ndarray:
         DESCRIPTION.
 
     """
-    kernel = []
-    for i in range(kernel_size//2):
-        kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
-        kernel_temp[i:kernel_size-i, i:kernel_size-i] = 1.0              
-        kernel.append(kernel_temp)
-    kernel = np.array(kernel)
-    
+    kernel = create_square_kernels(kernel_size=kernel_size)
+
     kernel2 = []
-    for i in range(kernel_size//2):   
+    for i in range(kernel_size//2):
         kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
         kernel_temp[kernel_size//2:kernel_size-i, kernel_size//2:kernel_size-i] = 1.0
         kernel2.append(kernel_temp)
@@ -416,7 +531,7 @@ def create_kuwahara_kernels(kernel_size=9) -> np.ndarray:
 
 def create_nagano_matsuyama_kernels(kernel_size=9) -> np.ndarray:
     """
-    
+    Create and stack of nagano-matsuyama kernels.
 
     Parameters
     ----------
@@ -429,16 +544,10 @@ def create_nagano_matsuyama_kernels(kernel_size=9) -> np.ndarray:
         DESCRIPTION.
 
     """
-    kernel = []
-    for i in range(kernel_size//2):
-        kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
-        kernel_temp[i:kernel_size-i, i:kernel_size-i] = 1.0  
-        kernel.append(kernel_temp)
-    kernel = np.array(kernel)
-
+    kernel = create_square_kernels(kernel_size=kernel_size)
 
     kernel2 = []
-    for i in range(kernel_size//2-1):   
+    for i in range(kernel_size//2-1):
         kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
         kernel_temp[kernel_size//2, kernel_size//2] = 1.0
         kernel_temp[kernel_size//2-1:kernel_size//2+2, kernel_size//2+1:kernel_size//2+3+i] = 1.0
@@ -452,7 +561,7 @@ def create_nagano_matsuyama_kernels(kernel_size=9) -> np.ndarray:
     kernel2 = np.array(kernel2)
 
     kernel3 = []
-    for i in range(kernel_size//2-1): 
+    for i in range(kernel_size//2-1):
         kernel_temp = np.full((kernel_size,  kernel_size), fill_value=0.0)
         for j in range(2+i):
             kernel_temp[kernel_size//2+j:kernel_size//2+2+j,
@@ -462,9 +571,9 @@ def create_nagano_matsuyama_kernels(kernel_size=9) -> np.ndarray:
         kernel_temp = np.flip(kernel_temp, axis=0)
         kernel3.append(kernel_temp)
         kernel_temp = np.flip(kernel_temp, axis=1)
-        kernel3.append(kernel_temp)       
+        kernel3.append(kernel_temp)
     kernel3 = np.array(kernel3)
-    
+
     return np.vstack([kernel, kernel2, kernel3])
 
 
@@ -485,18 +594,18 @@ class FilterKernel:
         if not file_path.is_file:
             raise FileNotFoundError('Configureation file not found')
         return str(file_path)
-        
+
     @staticmethod
     def read_configuration_files(*files):
         """
         Read .ini files using the configparser package.
-    
+
         Parameters
         ----------
         files : 'list' of 'str'
             List of file names of initialization files to be read to initialize
             an instance of a TSO object.
-    
+
         Raises
         ------
         ValueError
@@ -507,7 +616,7 @@ class FilterKernel:
         found = parser.read(files)
         if not found:
             raise ValueError('Config file not found!')
-            
+
         parameters = {}
         section_names = parser.sections()
         for name in section_names:
@@ -521,7 +630,7 @@ class FilterKernel:
         valid_model_dictionary =\
             {"anisotropic_gaussian": self.generate_anisotropic_gaussian_kernel,
              "banana": self.generate_banana_kernel,
-             "kuwahara": self.generate_kuwahara_kernel, 
+             "kuwahara": self.generate_kuwahara_kernel,
              "nagano_matsuyama": self.generate_nagano_matsuyama_kernel
         }
         return valid_model_dictionary
@@ -533,24 +642,24 @@ class FilterKernel:
                              "mininmum angle.")
         if np.abs(max_angle - min_angle) > 180.0:
             raise ValueError("The maximum angle range should not be larger "
-                             "than 180 degrees.") 
+                             "than 180 degrees.")
         if number_of_angles%2 - 1:
             warnings.warn("Number angles should be odd, adding 1")
             number_of_angles += 1
         return True
-        
+
     @staticmethod
     def check_kernel_shape(kernel_shape):
-        
+
         valid_2d_shapes = [(i, i) for i in IMPLEMENTED_KERNEL_SIZES]
         valid_3d_shapes = [(i, i, i) for i in IMPLEMENTED_KERNEL_SIZES]
-        
+
         if not ((kernel_shape in valid_2d_shapes) |
                 (kernel_shape in valid_3d_shapes)):
             raise ValueError("Kernel size not implemented. The following "
                              f"sizes can be used: {IMPLEMENTED_KERNEL_SIZES}."
             )
- 
+
         kernel_size = kernel_shape[0]
         kernel_dimension = len(kernel_shape)
 
@@ -560,7 +669,7 @@ class FilterKernel:
     def check_kernel_std(wavelength_stdev, spatial_stdev,
                          time_stdev=None):
         """
-        
+        Check the limits of the standard deviation of the used kernel.
 
         Parameters
         ----------
@@ -616,7 +725,7 @@ class FilterKernel:
 
     def generate_anisotropic_gaussian_kernel(self):
         """
-        
+        Generate an anisotropic gaussian kernel
 
         Returns
         -------
@@ -626,12 +735,12 @@ class FilterKernel:
         """
         kernel_size, kernel_dimension = \
             self.check_kernel_shape(self.configuration['kernel_shape'])
-        
+
         kernel_stack = \
             np.zeros((self.configuration['kernel_rotation_angles']['ntheta'],) +
                      self.configuration['kernel_shape']
                      )
-        
+
         theta_min = self.configuration['kernel_rotation_angles']['theta_min']
         theta_max = self.configuration['kernel_rotation_angles']['theta_max']
         ntheta = self.configuration['kernel_rotation_angles']['ntheta']
@@ -647,21 +756,21 @@ class FilterKernel:
         else:
             self.check_kernel_std(y_stddev, x_stddev)
             sigma = [y_stddev, x_stddev]
-        
+
         for ik, omega in enumerate(filter_angles):
             kernel = \
                 define_anisotropic_gaussian_kernel(
                     self.configuration['kernel_shape'], sigma=sigma,
-                    theta=[omega], degrees=False) 
-            
-            kernel_stack[ik, ...] = kernel     
+                    theta=[omega], degrees=False)
+
+            kernel_stack[ik, ...] = kernel
 
         if self.configuration['kernel_add_endpoints']:
 
             omega_endpoints = [0.0, -60.0, -45.0, 45.0, 60.0, 90.0]
             x_stddev_endpoint = [2.0,] +  [x_stddev,]*5
             y_stddev_endpoint = [2.0,] + [y_stddev,]*5
-            
+
             if  kernel_dimension == 3:
                 z_stddev_endpoint = [z_stddev, ]*6
                 sigma_endpoint = [[z,y,x] for z,y,x in zip(z_stddev_endpoint,
@@ -674,20 +783,20 @@ class FilterKernel:
             kernel_stack_endpoint = np.zeros((len(omega_endpoints),) +
                                              self.configuration['kernel_shape']
                      )
-        
+
             for ik, (omega, sigma) in enumerate(zip(omega_endpoints, sigma_endpoint)):
                    kernel = \
                        define_anisotropic_gaussian_kernel(
                            self.configuration['kernel_shape'], sigma=sigma,
-                           theta=[omega], degrees=True) 
+                           theta=[omega], degrees=True)
                    kernel_stack_endpoint[ik, ...] = kernel
-         
+
             kernel_stack = np.vstack([kernel_stack, kernel_stack_endpoint])
         return kernel_stack
 
     def create_kernel_stack(self):
         """
-        
+        Generate an kernel stack.
 
         Returns
         -------
